/*= driver gx310/320 ======================================================*/
/* Original Release: 13/10/2008                                            */
/* By: O.L.                                                                */
/* Originally written in C                                                 */
/*=========================================================================*/

#include "rs232.h"
#include "formatio.h"
#include "utility.h"
#include "gx3x0.h"

#define TEMPO	0.1

/*= STATIC VARIABLES ======================================================*/


/*= UTILITY ROUTINES ======================================================*/
int sendData (int serialPort, char *commande);
int readData (int serialPort, char *buff);


/*=========================================================================*/
/* Cette fonction ouvre le port srie , interroge l'appareil connect au   */
/* PC, reset l'appareil.                                                   */
/*=========================================================================*/
int gx3x0_init (int serialPort, int IDQuery, int resetDevice)
{
	int gx3x0_err = 0;
	char buff[50];

    gx3x0_err = OpenComConfig (serialPort, "", 19200, 0, 8, 1, 512, 512);
    
    if (rs232err == 0)
    	{
	    SetComTime (serialPort, 1.0);
	    SetXMode (serialPort, 0);
		SetCTSMode (serialPort, LWRS_HWHANDSHAKE_CTS_RTS);
		
		if (IDQuery)
			{
			gx3x0_err = sendData (serialPort, "*IDN?\r");
			gx3x0_err = readData (serialPort, buff);
			if (FindPattern (buff, 0, -1, "METRIX GX310", 0, 0) == -1 &&
				FindPattern (buff, 0, -1, "METRIX GX320", 0, 0) == -1)
				gx3x0_err = -1;
			}

		if (resetDevice)
			{
			gx3x0_err = sendData (serialPort, "*RST\r");
			}
        }
    else
		gx3x0_err = rs232err;
		
	return gx3x0_err;
}


/*=========================================================================*/
/* Cette fonction configure le type de signal, l'amplitude, l'offset,      */
/* le rapport cyclique, les niveaux logiques					           */
/*=========================================================================*/
int gx3x0_config (int serialPort, int function, double frequency, double amplitude, 
			double offset, int duty, double logicHigh, double logicLow)
{
	int gx3x0_err = 0;
	static char *Function[5] = {"SIN","TRI","SQU","LOGIC","DC"};
	char message[200];

	Fmt (message, "FUNC %s\r", Function[function]);
	gx3x0_err = sendData (serialPort, message);

	if (function == 4)
		{
		Fmt (message, "VOLT:OFFS %f\r", offset);
		gx3x0_err = sendData (serialPort, message);
		}
	else
		{	
		Fmt (message, "FREQ %f\r", frequency);
		gx3x0_err = sendData (serialPort, message);

		if (function)
			{
			Fmt (message, "PULS:DCYC %i\r", duty);
			gx3x0_err = sendData (serialPort, message);
			}

		if (function != 3)
			{
			Fmt (message, "VOLT %f\r", amplitude);
			gx3x0_err = sendData (serialPort, message);
			Fmt (message, "VOLT:OFFS %f\r", offset);
			gx3x0_err = sendData (serialPort, message);
			}
		else
			{
			Fmt (message, "VOLT:HIGH %f\r", logicHigh);
			gx3x0_err = sendData (serialPort, message);
			Fmt (message, "VOLT:LOW %f\r", logicLow);
			gx3x0_err = sendData (serialPort, message);
			}
		}
	
	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure les diffrentes fonctions du gnrateur        */
/* Cont, FSK, PSK, Sweep, Modul AM, modul FM, Frequencemetre, Syncro, Burst*/
/*=========================================================================*/
int gx3x0_function (int serialPort, int function)
{
	int gx3x0_err = 0;
	static char *Function[10] = {"CONT","FSK","PSK","SWE","AM","FM","FREQ",
								 "SYNCM","SYNCS","BURST"};
	char message[200];

	Fmt (message, "DEV:MOD %s\r", Function[function]);
	gx3x0_err = sendData (serialPort, message);
	
	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure le FSK, interne ou externe,                    */
/* la frequence Start et Stop                              				   */
/*=========================================================================*/
int gx3x0_fsk (int serialPort, double startFrequency, double stopFrequency, int source)
{
	int gx3x0_err = 0;
	static char *Source[2] = {"INT","EXT"};
	char message[200];
	
	Fmt (message, "FREQ:STAR %f\r", startFrequency);
	gx3x0_err = sendData (serialPort, message);
	Fmt (message, "SHIFT:SOUR %s\r", Source[source]);
	gx3x0_err = sendData (serialPort, message);
	
	if (source == 0)
		{
		Fmt (message, "FREQ:STOP %f\r", stopFrequency);
		gx3x0_err = sendData (serialPort, message);
		}

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure le PSK, interne ou externe,                    */
/* la Phase Start et Stop                               			   	   */
/*=========================================================================*/
int gx3x0_psk (int serialPort, int startPhase, int stopPhase, int source)
{
	int gx3x0_err = 0;
	static char *Source[2] = {"INT","EXT"};
	char message[200];
	
	Fmt (message, "SHIFT:PHAS %i\r", startPhase);
	gx3x0_err = sendData (serialPort, message);
	Fmt (message, "SHIFT:SOUR %s\r", Source[source]);
	gx3x0_err = sendData (serialPort, message);
	
	if (source == 0)
		{
		Fmt (message, "SHIFT:PHAS:STOP %i\r", stopPhase);
		gx3x0_err = sendData (serialPort, message);
		}

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure le Sweep, interne ou externe, lineaire ou      */
/* logarythmique, la dure, la frequence max, triangulaire ou dent de scie */
/*=========================================================================*/
int gx3x0_sweep (int serialPort, double startFrequency, double stopFrequency, double time, 
					int mode, int type, int source)
{
	int gx3x0_err = 0;
	static char *Source[2] = {"INT","EXT"};
	static char *Mode[2] = {"LIN","LOG"};
	static char *Type[2] = {"SAW","TRI"};
	char message[200];
	
	Fmt (message, "FREQ:STAR %f\r", startFrequency);
	gx3x0_err = sendData (serialPort, message);
	Fmt (message, "SWE:SOUR %s\r", Source[source]);
	gx3x0_err = sendData (serialPort, message);
		
	if (source == 0)
		{
		Fmt (message, "FREQ:STOP %f\r", stopFrequency);
		gx3x0_err = sendData (serialPort, message);
		Fmt (message, "SWE:TIME:LLIM %f\r", time);
		gx3x0_err = sendData (serialPort, message);
		Fmt (message, "SWE:SPAC %s\r", Mode[mode]);
		gx3x0_err = sendData (serialPort, message);
		Fmt (message, "SWE:TYP %s\r", Type[type]);
		gx3x0_err = sendData (serialPort, message);
		}

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure la modulation AM						       */
/* 20% ou 80%, et la source                                                */
/*=========================================================================*/
int gx3x0_am (int serialPort, int type, int source)
{
	int gx3x0_err = 0;
	static char *Source[2] = {"INT","EXT"};
	static int Type[2] = {80,20};
	char message[200];
	
	Fmt (message, "AM:SOUR %s\r", Source[source]);
	gx3x0_err = sendData (serialPort, message);
		
	if (source == 0)
		{
		Fmt (message, "AM %i\r", Type[type]);
		gx3x0_err = sendData (serialPort, message);
		}

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure la modulation FM						       */
/* 20% ou 80%, et la source                                                */
/*=========================================================================*/
int gx3x0_fm (int serialPort, double startFrequency, double stopFrequency, int source)
{
	int gx3x0_err = 0;
	static char *Source[2] = {"INT","EXT"};
	char message[200];
	
	Fmt (message, "FM:SOUR %s\r", Source[source]);
	gx3x0_err = sendData (serialPort, message);
		
	if (source == 0)
		{
		Fmt (message, "FREQ:STAR %f\r", startFrequency);
		gx3x0_err = sendData (serialPort, message);

		Fmt (message, "FREQ:STOP %f\r", stopFrequency);
		gx3x0_err = sendData (serialPort, message);
		}

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure la phase de la synchronisation			       */
/*                                                                         */
/*=========================================================================*/
int gx3x0_sync (int serialPort, int phase)
{
	int gx3x0_err = 0;
	char message[200];
	
	Fmt (message, "PHAS %i\r", phase);
	gx3x0_err = sendData (serialPort, message);
		
	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure le Burst, la dure, le nombre et la source     */
/* 																		   */
/*=========================================================================*/
int gx3x0_burst (int serialPort, double time, int num, int source)
{
	int gx3x0_err = 0;
	static char *Source[2] = {"INT","EXT"};
	char message[200];
	
	Fmt (message, "PULS:COUN %i\r", num);
	gx3x0_err = sendData (serialPort, message);
	Fmt (message, "PULS:SOUR %s\r", Source[source]);
	gx3x0_err = sendData (serialPort, message);
		
	if (source == 0)
		{
		Fmt (message, "PULS:DEL %f\r", time);
		gx3x0_err = sendData (serialPort, message);
		}

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction permet de lire la valeur du frequencemetre               */
/*																		   */
/*=========================================================================*/
int gx3x0_measure (int serialPort, double *frequency)
{
	int gx3x0_err = 0;
	char buff[100];

	gx3x0_err = sendData (serialPort, "MEAS?\r");
	gx3x0_err = readData (serialPort, buff);
	Scan (buff, "%f", frequency);

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure la sortie Main Out						       */
/* 																		   */
/*=========================================================================*/
int gx3x0_mainout (int serialPort, int mainout)
{
	int gx3x0_err = 0;
	char message[200];
	
	Fmt (message, "OUTP %i\r", mainout);
	gx3x0_err = sendData (serialPort, message);

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction configure le Gate sur la sortie					       */
/* 																		   */
/*=========================================================================*/
int gx3x0_gate (int serialPort, int gate)
{
	int gx3x0_err = 0;
	char message[200];
	
	Fmt (message, "OUTP:GATE %i\r", gate);
	gx3x0_err = sendData (serialPort, message);

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction active le Burst manuellement						       */
/* 																		   */
/*=========================================================================*/
int gx3x0_burstManTrig (int serialPort)
{
	int gx3x0_err = 0;
	char message[200];
	
	Fmt (message, "PULS:START\r");
	gx3x0_err = sendData (serialPort, message);

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction gre les mmoires de configurations           	       */
/* 																		   */
/*=========================================================================*/
int gx3x0_setup (int serialPort, int mem, int num)
{
	int gx3x0_err = 0;
	static char *Memory[3] = {"STOR:STAT","LOAD:STAT","DEL"};
	char message[200];
	
	Fmt (message, "MMEM:%s %i\r", Memory[mem], num);
	gx3x0_err = sendData (serialPort, message);

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction permet de couper/demarrer le gnrateur			       */
/* 																		   */
/*=========================================================================*/
int gx3x0_power (int serialPort, int power)
{
	int gx3x0_err = 0;
	char message[200];
	
	Fmt (message, "SYST:POW %i\r", power);
	gx3x0_err = sendData (serialPort, message);

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette fonction permet de regler le contraste de l'ecran  		       */
/* 																		   */
/*=========================================================================*/
int gx3x0_contrast (int serialPort, int contrast)
{
	int gx3x0_err = 0;
	char message[200];
	
	Fmt (message, "DISP:CONT %f\r", contrast / 100.0);
	gx3x0_err = sendData (serialPort, message);

	return gx3x0_err;
}

/*=========================================================================*/
/* Cette function ferme le port srie                                      */
/*                                                                         */
/*=========================================================================*/
int gx3x0_close (int serialPort)
{
	int gx3x0_err = 0;

    gx3x0_err = CloseCom (serialPort);
    
	return gx3x0_err;
}


/*=========================================================================*/
/* cette fonction crit dans un buffer de data la chaine de commande       */
/*  envoyer                                                               */
/*=========================================================================*/
int sendData (int serialPort, char *commande)
{
	int gx3x0_err = 0;

	FlushInQ (serialPort);
	FlushOutQ (serialPort);
    ComWrt (serialPort, commande, StringLength (commande));
    Delay (TEMPO);
    
    if (rs232err != 0)
		gx3x0_err = rs232err;
		
	return gx3x0_err;
}


/*=========================================================================*/
/* cette fonction lit dans un buffer la chaine de commande renvoye        */
/*                                                                         */
/*=========================================================================*/
int readData (int serialPort, char *buff)
{
	int gx3x0_err = 0;

    ComRdTerm (serialPort, buff, 50, 0x0D);

    if (rs232err != 0)
		gx3x0_err = rs232err;
		
	return gx3x0_err;
}

